@@ -232,13 +232,17 @@ public class Widget extends GtkObject {
     }
 
     /**
-     * Return the widget's gdk window. This is the window to draw upon when
-     * writing custom widgets. It's available after the widget has been mapped.
-     * Thus if you listen to LifeCycleEvents, you can fetch the Window instance
-     * using this method after an LifeCycleEvent of type MAP has occurred.
+     * Return the widget's Gdk window, <i>if it has one</i>. This is the window
+     * to draw upon when writing custom widgets. It's available after the widget
+     * has been mapped. Thus if you listen to LifeCycleEvents, you can fetch the
+     * Window instance using this method after an LifeCycleEvent of type MAP has
+     * occurred.
      * 
-     * @return the widget's gdk window, or null if a) it hasn't been mapped yet
-     *         or b) it is of a type that doesn't have a gdk window.
+     * @return the widget's Gdk Window, or null if a) it hasn't been mapped yet
+     *         or b) it is of a type that doesn't have a Gdk Window.
+     * @see #getParentWindow() if you want to be sure of getting to a Gdk
+     *      Window.
+     * @see DrawingArea for more about custom Widgets.
      */
     public Window getWindow() {
         checkState();
@@ -329,6 +333,34 @@ public class Widget extends GtkObject {
         Widget.gtk_widget_grab_focus(getHandle());
     }
 
+    /**
+     * Find out the current value of the "can-focus" property
+     * 
+     * @since 2.8.5
+     * @see #setCanFocus(boolean)
+     */
+    public boolean getCanFocus() {
+        checkState();
+        return getBooleanProperty("can-focus");
+    }
+
+    /**
+     * Tell this Widget whether or not it is allowed to accept focus. Telling a
+     * widget that it is not to accept focus generally means that it will be
+     * skipped over when TABing around a Window. This can, however, have some
+     * unexpected side effects: if a Label, for example, is told that it can't
+     * have focus but it also told that its text is
+     * {@link Label#setSelectable(boolean) selectable}, then you won't be able
+     * to Copy and Paste any text you select from it.
+     * 
+     * @param canFocus
+     * @since 2.8.5
+     */
+    public void setCanFocus(boolean canFocus) {
+        checkState();
+        setBooleanProperty("can-focus", canFocus);
+    }
+
     /**
      * Causes the widget to become the default widget. The default widget is
      * activated when the user presses Enter in the window. Default widgets must
@@ -703,6 +735,14 @@ public class Widget extends GtkObject {
         return Display.getDisplayFromHandle(hndl);
     }
 
+    /**
+     * Get the root window where this widget is located. This function can only
+     * be called after the widget has been added to a widget heirarchy with Gtk
+     * {@link org.gnu.gtk.Window} at the top.
+     * 
+     * @return the Gdk Window backing the root of this particular Widget's
+     *         hierarchy.
+     */
     public Window getRootWindow() {
         checkState();
         Handle hndl = gtk_widget_get_root_window(getHandle());
@@ -1360,6 +1400,10 @@ public class Widget extends GtkObject {
                 wname = "StatusBar";
             } else if (wname.equals("Toolbar")) {
                 wname = "ToolBar";
+            } else if (wname.equals("HScrollbar")) {
+                wname = "HScrollBar";
+            } else if (wname.equals("VScrollbar")) {
+                wname = "VScrollBar";
             }
             return "org.gnu.gtk." + wname;
         } else if (widgetType.startsWith("Gnome"))
@@ -1583,7 +1627,8 @@ public class Widget extends GtkObject {
                 dragOriginListeners = new Vector();
                 evtMap.initialize(this, StartDragEvent.Type.START);
                 evtMap.initialize(this, EndDragEvent.Type.END);
-                evtMap.initialize(this, RequestDragDataEvent.Type.DATA_REQUEST, false);
+                evtMap.initialize(this, RequestDragDataEvent.Type.DATA_REQUEST,
+                        false);
                 evtMap.initialize(this, DeleteDragDataEvent.Type.DATA_DELETE);
             }
             dragOriginListeners.addElement(listener);
